package be.iminds.ilabt.jfed.util.sentry;

import be.iminds.ilabt.jfed.lowlevel.user.GeniUser;
import be.iminds.ilabt.jfed.preferences.JFedPreferences;
import be.iminds.ilabt.jfed.util.library.JFedVersionInfo;
import be.iminds.ilabt.jfed.util.library.KeyUtil;
import io.sentry.Sentry;
import io.sentry.event.UserBuilder;

import javax.annotation.Nullable;
import java.io.IOException;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.util.List;

public class SentryUtil {
    //hint: CTRL-F4 to test uncaught exception

    public static void setUserContext(@Nullable GeniUser geniUser) {
        if (geniUser != null) {
            Sentry.getContext().setUser(
                    new UserBuilder()
                            .setId(geniUser.getUserUrnString())
                            .setEmail(geniUser.getCertEmail())
                            .withData("expire", geniUser.getCertExpire())
                            .withData("urn", geniUser.getUserUrnString())
                            .withData("cert", geniUser.getClientCertificateChain() != null ? KeyUtil.x509certificateChainToPem(geniUser.getClientCertificateChain()) : null)
                            .setUsername(geniUser.getUserUrn() != null ? geniUser.getUserUrn().getResourceName() : null)
                            .build()
            );
        } else {
            Sentry.getContext().clearUser();
        }
    }

    //Sentry docs say: "Releases are global per organization, so make sure to prefix them with something project-specific."
    public static final String SENTRY_RELEASE_PREFIX = "jfed@";

    public static void setVersionContext(JFedVersionInfo versionInfo) {
        boolean isFromIDE = false;

        if (versionInfo.getGitBranch().startsWith("${env.")) {
            Sentry.getContext().addTag("git.branch", versionInfo.getGitBranch());
            isFromIDE= true;
        }
        if (versionInfo.getGitCommit().startsWith("${env.")) {
            Sentry.getContext().addTag("git.commit", versionInfo.getGitCommit());
            isFromIDE= true;
        }
        if (versionInfo.getJenkinsBuild().startsWith("${env.")) {
            Sentry.getContext().addTag("build", versionInfo.getJenkinsBuild());
            isFromIDE= true;
        }
        if (versionInfo.getVersion().startsWith("${")) {
            Sentry.getContext().addTag("jfed.version", versionInfo.getVersion());
            isFromIDE= true;
        }

        boolean isSnapshot = versionInfo.getVersion().contains("-SNAPSHOT"); //example: "6.3.3-SNAPSHOT"

        if (isFromIDE) {
            Sentry.getContext().addTag("environment", "dev");
        } else if (isSnapshot) {
            Sentry.getContext().addTag("environment", "snapshot");
            Sentry.getContext().addTag("release", SENTRY_RELEASE_PREFIX+versionInfo.getVersion());
        } else {
            Sentry.getContext().addTag("environment", "production");
            Sentry.getContext().addTag("release", SENTRY_RELEASE_PREFIX+versionInfo.getVersion());
        }

        //All data in "environment_string" This is typically auto captured by sentry, but apparently not on java
        Sentry.getContext().addExtra("environment_string", JFedVersionInfo.getEnvironmentString());
        Sentry.getContext().addTag("device", System.getProperty("os.arch"));
        Sentry.getContext().addTag("OS", System.getProperty("os.name")+" "+System.getProperty("os.version"));
        Sentry.getContext().addTag("java.vendor", System.getProperty("java.vendor"));
        Sentry.getContext().addTag("java.version", System.getProperty("java.version"));
        Sentry.getContext().addTag("runtime","java " + System.getProperty("java.version"));
    }

    public static String createSentryPreferences(JFedPreferences jFedPreferences) {
        if (jFedPreferences.getPropertiesFile() != null) {
            List<String> lines = null;
            try {
                lines = Files.readAllLines(jFedPreferences.getPropertiesFile().toPath(), Charset.defaultCharset());

                StringBuilder preferencesString = new StringBuilder();
                for (String line : lines) {
                    preferencesString.append(line).append("\n");
                }
                return preferencesString.toString();
            } catch (IOException e) {
                return "Exception reading user preferences file in createSentryPreferences(): "+e.getMessage();
            }
        } else {
            return "No properties file. (User without home dir?)\n";
        }
    }
}
